//+------------------------------------------------------------------+
//|                                                          CSM.mq4 |
//+------------------------------------------------------------------+

#property indicator_separate_window

#include <hanover --- function header.mqh>

//extern string   Currencies          = "AUD,CAD,CHF,EUR,GBP,JPY,NZD,USD,TRY,SGD,DKK,HKD,NOK,SEK,PLN,HUF,CZK,ZAR";
//extern string   CurrencyPairs       = "AUD,CAD,CHF,EUR,GBP,JPY,NZD,USD";
extern string   CurrencyPairs       = "GU,UF,EU,UJ,UC,NU,AU,AN,AC,AF,AJ,CJ,FJ,EG,EA,EF,EJ,EN,EC,GF,GA,GC,GJ,GN,NJ,NC,CF,NF";
extern string   CurrencySuffix      = "";
extern string   TimeFrames          = "M1,M5,M15,M30,H1,H4,D1,W1,MN";
extern int      NumberOfCandles     = 25;
extern bool     ApplySmoothing      = true;
extern bool     TriangularWeighting = false;
extern int      HistoricalShift     = 0;
extern string   SortMethod          = "D";
extern bool     ShowValues          = true;
extern string   FontName            = "Placard MT Condensed";
extern int      FontSize            = 12;
extern bool     SeparateWindow      = true;
extern int      HorizPos            = 30;
extern int      VertPos             = 180;
extern int      HorizSpacing        = 15;
//extern string   UseAltColors        = "DodgerBlue,25,Yellow,50,Orange,75,Red";
extern string   UseAltColors        = "";
extern color    Color_AUD           = Green;
extern color    Color_CAD           = SaddleBrown;
extern color    Color_CHF           = Yellow;
extern color    Color_EUR           = HotPink;
extern color    Color_GBP           = Crimson;
extern color    Color_JPY           = DarkOrchid;
extern color    Color_NZD           = CornflowerBlue;
extern color    Color_USD           = White;
extern string   TextFont            = "Verdana";
extern int      TextSize            = 12;
extern color    TextColor           = Silver;
extern string   RefreshPeriod       = "M1";
extern string   UniqueID            = "1";
extern string   OutputFilename      = "CSM.CSV";
extern bool     HighlightBox        = true;

double    spr, pnt, tickval, ccy_strength[8];
int       dig, tf, ccy_count[8], ccCP, wno, RefreshEveryXMins, nAltColors;
string    IndiName, ccy, CP[99], ccy_name[8];
datetime  prev_time;
color     colr[8];
string    altclr[20];


//+------------------------------------------------------------------+
int init()   {
//+------------------------------------------------------------------+
  RefreshEveryXMins = StrToTF(RefreshPeriod);

  NumberOfCandles = MathMax(NumberOfCandles,1);
  
  if (CurrencyPairs == "")  CurrencyPairs = Symbol();
  CurrencyPairs  = AppendIfMissing(StringUpper(CurrencyPairs),",");
  ccCP = StrToStringArray(CurrencyPairs,CP);
  for (int i=0; i<=ccCP; i++)
    CP[i] = ExpandCcy(CP[i]) + CurrencySuffix;

  if (TimeFrames == "")  TimeFrames = TFToStr(Period());
  TimeFrames  = AppendIfMissing(StringUpper(TimeFrames),",");

  nAltColors = StrToStringArray(UseAltColors,altclr);
  
  IndiName = "CSM-" + UniqueID + "-";
  IndicatorShortName(IndiName);

  ccy     = Symbol();
  tf      = Period();
  pnt     = MarketInfo(ccy,MODE_POINT);
  dig     = MarketInfo(ccy,MODE_DIGITS);
  spr     = MarketInfo(ccy,MODE_SPREAD);
  tickval = MarketInfo(ccy,MODE_TICKVALUE);
  if (dig == 3 || dig == 5) {
    pnt     *= 10;
    spr     /= 10;
    tickval *= 10;
  }

  ccy_name[0] = "AUD";    colr[0] = Color_AUD;
  ccy_name[1] = "CAD";    colr[1] = Color_CAD;
  ccy_name[2] = "CHF";    colr[2] = Color_CHF;
  ccy_name[3] = "EUR";    colr[3] = Color_EUR;
  ccy_name[4] = "GBP";    colr[4] = Color_GBP;
  ccy_name[5] = "JPY";    colr[5] = Color_JPY;
  ccy_name[6] = "NZD";    colr[6] = Color_NZD;
  ccy_name[7] = "USD";    colr[7] = Color_USD;

  del_obj();
  plot_obj();    
  prev_time = -9999;

  wno = WindowFind(IndiName);
  if (wno < 0)
    wno = WindowsTotal();
  if (!SeparateWindow)
    wno = 0;  

  return(0);
}

//+------------------------------------------------------------------+
int deinit()  {
//+------------------------------------------------------------------+
  del_obj();
  return(0);
}

//+------------------------------------------------------------------+
int start()  {
//+------------------------------------------------------------------+
  if (RefreshEveryXMins == 0) {
    del_obj();
    plot_obj();    
  }
  else {
    if(prev_time != iTime(Symbol(),RefreshEveryXMins,0))  {
      del_obj();
      plot_obj();
      prev_time = iTime(Symbol(),RefreshEveryXMins,0);
  } }      
  return(0);
}

//+------------------------------------------------------------------+
void del_obj()   {
//+------------------------------------------------------------------+
  int k=0;
  while (k<ObjectsTotal())   {
    string objname = ObjectName(k);
    if (StringSubstr(objname,0,StringLen(IndiName)) == IndiName)  
      ObjectDelete(objname);
    else
      k++;
  }    
  return(0);
}

//+------------------------------------------------------------------+
void plot_obj()   {
//+------------------------------------------------------------------+
  if (HighlightBox)   {
    string objname = IndiName + "991";
    ObjectCreate(objname,OBJ_LABEL,wno,0,0);
    ObjectSet(objname,OBJPROP_XDISTANCE,781);
    ObjectSet(objname,OBJPROP_YDISTANCE,24);
    ObjectSetText(objname,StringRepeat("",8),18,"Arial",DimGray);
    ObjectSet(objname,OBJPROP_ANGLE,270.0);
  
    objname = IndiName + "992";
    ObjectCreate(objname,OBJ_LABEL,wno,0,0);
    ObjectSet(objname,OBJPROP_XDISTANCE,765);
    ObjectSet(objname,OBJPROP_YDISTANCE,200);
    ObjectSetText(objname,StringRepeat("",19),18,"Arial",DimGray);
    ObjectSet(objname,OBJPROP_ANGLE,0.0);

    objname = IndiName + "993";
    ObjectCreate(objname,OBJ_LABEL,wno,0,0);
    ObjectSet(objname,OBJPROP_XDISTANCE,1237);
    ObjectSet(objname,OBJPROP_YDISTANCE,22);
    ObjectSetText(objname,StringRepeat("",8),18,"Arial",DimGray);
    ObjectSet(objname,OBJPROP_ANGLE,270.0);
  }
  
  if (OutputFilename > "")   {
    int outf = FileOpen(OutputFilename,FILE_CSV|FILE_WRITE,'~');
    if (outf >= 0)  {     
      string outstr = "TF ";
      for (int i=0; i<8; i++)   {
        outstr = outstr + ", " + ccy_name[i];
      }
      FileWrite(outf,outstr);
  } }  
    
  int hi_bar, lo_bar;
  double curr_bid, candle_high, candle_low, bid_ratio;
  int xp = HorizPos;
  int yp = VertPos;
  ArrayInitialize(ccy_strength,0.0);
  ArrayInitialize(ccy_count,0);

  double CandleSum = NumberOfCandles*(NumberOfCandles+1)/2;
  for (int t=0; t<9; t++)  {
    int tmf = (t==0) + 5*(t==1) + 15*(t==2) + 30*(t==3) + 60*(t==4) + 240*(t==5) + 1440*(t==6) + 10080*(t==7) + 43200*(t==8);
    if (StringFind(TimeFrames,TFToStr(tmf)+",") < 0)    continue;
    ArrayInitialize(ccy_strength,0);
    ArrayInitialize(ccy_count,0);

    for (i=0; i<ccCP; i++)   {
      curr_bid     = MarketInfo(CP[i],MODE_BID);
      if (curr_bid == 0)   continue;
      
      curr_bid = iClose(CP[i],tmf,HistoricalShift);
      if (ApplySmoothing)   {
        bid_ratio = 0;
        for (int k=1; k<=NumberOfCandles; k++)  {
          hi_bar       = iHighest(CP[i],tmf,MODE_HIGH,k+HistoricalShift,HistoricalShift);
          lo_bar       = iLowest(CP[i],tmf,MODE_LOW,k+HistoricalShift,HistoricalShift);
          candle_high  = iHigh(CP[i],tmf,hi_bar);
          candle_low   = iLow(CP[i],tmf,lo_bar);
          if (TriangularWeighting)
            bid_ratio    += DivZero(curr_bid - candle_low, candle_high - candle_low) * (NumberOfCandles+1-k)/CandleSum;
          else
            bid_ratio    += DivZero(curr_bid - candle_low, candle_high - candle_low) / NumberOfCandles;
        }
      } else {
        hi_bar       = iHighest(CP[i],tmf,MODE_HIGH,NumberOfCandles+HistoricalShift,HistoricalShift);
        lo_bar       = iLowest(CP[i],tmf,MODE_LOW,NumberOfCandles+HistoricalShift,HistoricalShift);
        candle_high  = iHigh(CP[i],tmf,hi_bar);
        candle_low   = iLow(CP[i],tmf,lo_bar);
        bid_ratio    = DivZero(curr_bid - candle_low, candle_high - candle_low);
      }
//      log(CP[i],TFToStr(tmf),bid_ratio);
/*
      double ind_strength = 0;
      if (bid_ratio >= 0.97)   ind_strength = 9;    else
      if (bid_ratio >= 0.90)   ind_strength = 8;    else
      if (bid_ratio >= 0.75)   ind_strength = 7;    else
      if (bid_ratio >= 0.60)   ind_strength = 6;    else
      if (bid_ratio >= 0.50)   ind_strength = 5;    else
      if (bid_ratio >= 0.40)   ind_strength = 4;    else
      if (bid_ratio >= 0.25)   ind_strength = 3;    else
      if (bid_ratio >= 0.10)   ind_strength = 2;    else
      if (bid_ratio >= 0.03)   ind_strength = 1;
*/      
      double ind_strength = 10 * bid_ratio;

      string temp = StringSubstr(CP[i],0,3);
      for (int j=0; j<8; j++)   {
        if (ccy_name[j] == temp)  {
          ccy_strength[j] += ind_strength;
          ccy_count[j]    += 1;
          break;
      } }    

      temp = StringSubstr(CP[i],3,3);
      for (j=0; j<8; j++)   {
        if (ccy_name[j] == temp)  {
          ccy_strength[j] += 10 - ind_strength;
          ccy_count[j]    += 1;
          break;
    } } }    

    // This routine loads the strength values and currency symbols into an array, and sorts the array......
    string array_to_sort[8];
    for (j=0; j<8; j++)  {
      array_to_sort[j] = "";
//      if (ccy_count[j] < 1)   continue;
      double out_value = DivZero(ccy_strength[j],ccy_count[j]);
      string tstr1     = NumberToStr(out_value,"R-3.1") + ccy_name[j] + NumberToStr(colr[j],"12");
      array_to_sort[j] = tstr1;
    }
    if (StringUpper(SortMethod) == "D")     ShellsortStringArray(array_to_sort,true);
    if (StringUpper(SortMethod) == "A")     ShellsortStringArray(array_to_sort,false);
                                                                       
    for (j=0; j<8; j++)  {
      ccy        = StringSubstr(array_to_sort[j],6,3);
      out_value  = StrToNumber(StringSubstr(array_to_sort[j],0,6));
      color clr  = StrToNumber(StringSubstr(array_to_sort[j],9,12));
      if (nAltColors > 0)  {
        clr = StrToColor(altclr[0]);
        for (int cc=1; cc<nAltColors; cc+=2)  {
          if (out_value*10 >= StrToNumber(altclr[cc]))   clr = StrToColor(altclr[cc+1]);
      } }      
      tstr1     = StringRepeat("I",MathFix(5*out_value,0));
      objname   = IndiName + "-" + TFToStr(tmf) + "-" + ccy; 
      ObjectCreate(objname,OBJ_LABEL,wno,0,0);
      ObjectSet(objname,OBJPROP_XDISTANCE,xp);
      ObjectSet(objname,OBJPROP_YDISTANCE,yp);
      ObjectSetText(objname,tstr1,FontSize,FontName,clr);
      ObjectSet(objname,OBJPROP_ANGLE,90.0);
//      log(i,j,objname,out_value,tstr1);

      objname   = IndiName + "-" + TFToStr(tmf) + "-" + ccy + "-CCY"; 
      ObjectCreate(objname,OBJ_LABEL,wno,0,0);
      ObjectSet(objname,OBJPROP_XDISTANCE,xp+5);
      ObjectSet(objname,OBJPROP_YDISTANCE,yp+28);
      ObjectSetText(objname,ccy,8,TextFont,clr);
      ObjectSet(objname,OBJPROP_ANGLE,90.0);

      if (j==4)   {
        objname   = IndiName + "-" + TFToStr(tmf) + "-LBL"; 
        ObjectCreate(objname,OBJ_LABEL,wno,0,0);
        ObjectSet(objname,OBJPROP_XDISTANCE,xp-HorizSpacing/2);
        ObjectSet(objname,OBJPROP_YDISTANCE,yp+37);
        ObjectSetText(objname,TFToStr(tmf),TextSize,TextFont,TextColor);
      }
      if (ShowValues)   {
        objname   = IndiName + "-" + TFToStr(tmf) + "-" + ccy + "-VAL"; 
        ObjectCreate(objname,OBJ_LABEL,wno,0,0);
        ObjectSet(objname,OBJPROP_XDISTANCE,xp+HorizSpacing/2-3);
        ObjectSet(objname,OBJPROP_YDISTANCE,yp-15*out_value-10);
        ObjectSetText(objname,NumberToStr(10*out_value,"TR3"),8,TextFont,clr);
        ObjectSet(objname,OBJPROP_ANGLE,90.0);
      }
      
      xp += HorizSpacing;
    }        
    xp += HorizSpacing*2;
    if (outf >= 0)  {     
      outstr = StrToStr(TFToStr(tmf),"L3");
      for (i=0; i<8; i++)   {
        outstr = outstr + NumberToStr(10*DivZero(ccy_strength[i],ccy_count[i]),"','R4");
      }  
      FileWrite(outf,outstr);
  } } 
  if (outf >= 0)       
    FileClose(outf);
  return(0);
}

#include <hanover --- extensible functions.mqh>

